{===================== DRAG AND DROP DESTINATION HELPERS ======================}
{                                                                              }
{ These components are intended to provide an easy way to implement drag and   }
{ drop of files into a Delphi application.                                     }
{                                                                              }
{ TDnDRun     - items which have been dropped on the application's icon on     }
{               either the desktop or filemanager/explorer                     }
{ TDnDForm    - items dropped on a form                                        }
{ TDnDControl - items dropped on a control (TWinControl only)                  }
{ Delphi 1 only:                                                               }
{ TDnDIcon    - dropped on a minimised application icon.                       }
{                                                                              }
{ Each component produces the same three events:                               }
{    DnDStart  - sent before the items dropped list is sent                    }
{    DnDItem   - sent for each item dropped                                    }
{    DnDFinish - sent when all items have been sent                            }
{                                                                              }
{ *Warning*  I have tested the code under Delphi 3 *ONLY*.  I have NOT tested  }
{  -------                                          ----           ---         }
{ it under Delphi 1 or Delphi 2.  Some parts of this code are Delphi 1,        }
{ infact one of the components will only appear under Delphi 1.  All I can say }
{ is that the code looks ok. Delphi 1 and 2 users use this code on this        }
{ understanding. I would appreciate comment that it is ok or corrections.      }
{                                                                              }
{ Version 1.00                                                                 }
{ Grahame Marsh 16 October 1997                                                }
{                                                                              }
{ Freeware - you get it for free, I take nothing, I make no promises!          }
{                                                                              }
{ Please feel free to contact me: grahame.s.marsh@courtaulds.com               }
{                                                                              }
{ Revison History:                                                             }
{    Version 1.00 - initial release  16-10-97                                  }
{                                                                              }
{==============================================================================}

unit DDDest;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, ShellAPI, DsgnIntf;

type
  TDnDStartEvent = procedure (Sender: TObject; Count, X, Y: integer) of object;
  TDnDItemEvent = procedure (Sender: TObject; const Item: string) of object;

  TDnDRun = class (TComponent)
  private
    FOnDnDStart: TDnDStartEvent;
    FOnDnDItem: TDnDItemEvent;
    FOnDnDFinish: TNotifyEvent;
  protected
    procedure Loaded; override;
  published
    property OnDnDStart: TDnDStartEvent read FOnDnDStart write FOnDnDStart;
    property OnDnDItem: TDnDItemEvent read FOnDnDItem write FOnDnDItem;
    property OnDnDFinish: TNotifyEvent read FOnDnDFinish write FOnDnDFinish;
  end;

  TDnDForm = class(TComponent)
  private
    FParent: THandle;
    FOlDnDefWndProc,
    FNewDefWndProc: pointer;
    FOnDnDStart: TDnDStartEvent;
    FOnDnDItem: TDnDItemEvent;
    FOnDnDFinish: TNotifyEvent;
    procedure NewDefWndProc (var Msg: TMessage);
  protected
    procedure Loaded; override;
  public
  published
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    property OnDnDStart: TDnDStartEvent read FOnDnDStart write FOnDnDStart;
    property OnDnDItem: TDnDItemEvent read FOnDnDItem write FOnDnDItem;
    property OnDnDFinish: TNotifyEvent read FOnDnDFinish write FOnDnDFinish;
  end;

  TDnDControl = class(TDnDForm)
  private
    FControl: TWinControl;
    procedure SetControl (Value: TWinControl);
  protected
    procedure Loaded; override;
{$IFDEF WIN32}
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
{$ENDIF}
  public
  published
    property Control: TWinControl read FControl write SetControl;
  end;

{$IFNDEF WIN32}
  TDnDIcon = class (TComponent)
  private
    FOldOnMessage: TMessageEvent;
    FOnDnDStart: TDnDStartEvent;
    FOnDnDItem: TDnDItemEvent;
    FOnDnDFinish: TNotifyEvent;
  protected
    procedure DropIconMessage (var Msg: TMsg; var Handled: boolean);
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property OnDnDStart: TDnDStartEvent read FOnDnDStart write FOnDnDStart;
    property OnDnDItem: TDnDItemEvent read FOnDnDItem write FOnDnDItem;
    property OnDnDFinish: TNotifyEvent read FOnDnDFinish write FOnDnDFinish;
  end;
{$ENDIF}

function DroppedCount (DropHandle: hDrop): integer;
function DroppedItem (DropHandle: hDrop; Item: integer): string;

procedure Register;

implementation

function DroppedCount (DropHandle: hDrop): integer;
begin
  Result := DragQueryFile (DropHandle, -1, nil, 0)
end;

function DroppedItem (DropHandle: hDrop; Item: integer): string;
{$IFNDEF WIN32}
const
  MAX_PATH = 127;
{$ENDIF}
begin
  SetLength (Result, MAX_PATH);
  DragQueryFile (DropHandle, Item, pchar(Result), MAX_PATH-1);
  SetLength (Result, StrLen(pchar(Result)))
end;

{=== DROPPED ON RUN ===========================================================}

procedure TDnDRun.Loaded;
var
  Loop: integer;
begin
  if ParamCount > 0 then
  begin
    if Assigned (FOnDnDStart) then
      FOnDnDStart (Self, ParamCount, 0, 0);

    if Assigned (FOnDnDItem) then
      for Loop := 1 to ParamCount do
        FOnDnDItem (Self, ParamStr (Loop));

    if Assigned (FOnDnDFinish) then
      FOnDnDFinish (Self)
  end
end;

{=== DROPPED ON FORM ==========================================================}

constructor TDnDForm.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FParent := (AOwner as TForm).Handle;
  FNewDefWndProc := MakeObjectInstance (NewDefWndProc);
end;

procedure TDnDForm.Loaded;
begin
  inherited Loaded;
  FOlDnDefWndProc := pointer(SetWindowLong (FParent, GWL_WndProc, longint(FNewDefWndProc)));
  DragAcceptFiles (FParent, true)
end;

destructor TDnDForm.Destroy;
begin
  SetWindowLong (FParent, GWL_WndProc, longint(FOlDnDefWndProc));
  FreeObjectInstance (FNewDefWndProc);
  inherited Destroy
end;

procedure TDnDForm.NewDefWndProc (var Msg: TMessage);
var
  Loop,
  Count: integer;
  Point: TPoint;
begin
  with Msg do
  begin
    if Msg = WM_DropFiles then
    begin
      DragQueryPoint (wParam, Point);

      Count:= DroppedCount (wParam);
      if Assigned (FOnDnDStart) then
        FOnDnDStart (Self, Count, Point.X, Point.Y);

      if Assigned (FOnDnDItem) then
        for Loop:= 0 to Count - 1 do
          FOnDnDItem (Self, DroppedItem (wParam, Loop));

      DragFinish (wParam);
      if Assigned (FOnDnDFinish) then
        FOnDnDFinish (Self);

      Result := 0;
      exit
    end else
      if Msg = WM_Destroy then
        DragAcceptFiles (FParent, false);

    Result := CallWindowProc (FOlDnDefWndProc, FParent, Msg, wParam, lParam)
  end
end;

{=== DROPPED ON A TWinControl =================================================}

procedure TDnDControl.Loaded;
begin
  FParent := FControl.Handle;
  inherited Loaded
end;

procedure TDnDControl.SetControl (Value: TWinControl);
begin
  FControl := Value;
{$IFDEF WIN32}
  if Value <> nil then
    Value.FreeNotification(Self)
{$ENDIF}
end;

{$IFDEF WIN32}
procedure TDnDControl.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification (AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FControl) then
    FControl := nil
end;
{$ENDIF}

{=== DROPPED ON AN ICON APP ===================================================}

{$IFNDEF WIN32}
constructor TDnDIcon.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FOldOnMessage := Application.OnMessage;
  Application.OnMessage := DropIconMessage
end;

destructor TDnDIcon.Destroy;
begin
  Application.OnMessage := FOldOnMessage;
  inherited Destroy
end;

procedure TDnDIcon.DropIconMessage (var Msg: TMsg; var Handled: boolean);
var
  Loop,
  Count: integer;
begin
  with Msg do
    if (Message = WM_DropFiles) and (Application.MainForm.WindowState = wsMinimized) then
    begin
      Count:= DroppedCount (wParam);
      if Assigned (FOnDnDStart) then
        FOnDnDStart (Self, Count, 0, 0);

      if Assigned (FOnDnDItem) then
        for Loop:= 0 to Count - 1 do
          FOnDnDItem (Self, DroppedItem (wParam, Loop));

      DragFinish (wParam);
      if Assigned (FOnDnDFinish) then
        FOnDnDFinish (Self);
      Handled := true
    end else
      if Assigned (FOldOnMessage) then
        FOldOnMessage (Msg, Handled)
end;
{$ENDIF}

procedure Register;
begin
  RegisterComponents ('My Controls', [TDnDRun, TDnDForm, TDnDControl]);
end;

end.


